import logging
import importlib
from rest_framework import serializers as rest_serializers
from rest_extensions.fields import DataListField
from rest_extensions.serializers import MyJSONField
from . import utils


class AutoSerializerMixin(object):
    def get_serializer_class(self):
        logging.info("get serializers")
        serializers = None
        if getattr(self, "serializers", None):
            logging.info("has serializers")
            serializers = self.serializers
        elif getattr(self, "app_module", None):
            logging.info("has app_module")
            if importlib.util.find_spec(
                    "{}.serializers".format(self.app_config.name)):
                serializers = importlib.import_module(
                    "{}.serializers".format(self.app_config.name))
        serializer_name1 = "{model_name}{action}Serializer".format(
            model_name=self.model.__name__,
            action=self.action.capitalize(),
        )
        serializer_name2 = "{model_name}Serializer".format(
            model_name=self.model.__name__, )
        serializer_name3 = "{model_name}BaseSerializer".format(
            model_name=self.model.__name__, )
        for name in [serializer_name1, serializer_name2, serializer_name3]:
            serializer_class = getattr(serializers, name, None)
            if serializer_class:
                return serializer_class
        logging.info("{} Does not exist".format(serializer_name1, serializer_name2, serializer_name3))

        if self.action == "create":
            return utils.CreateSerializerFactory(
                self.model).create_serializer()
        elif self.action == "list":
            return utils.ListSerializerFactory(
                self.model).create_serializer()
        elif "update" in self.action:
            return utils.UpdateSerializerFactory(
                self.model).create_serializer()

        class Meta:
            fields = "__all__"
            depth = 0
            model = self.model

        tmpMeta = Meta

        class TmpSerializer(rest_serializers.ModelSerializer):
            Meta = tmpMeta

        serializer_class = TmpSerializer
        serializer_class.serializer_field_mapping[DataListField] = MyJSONField
        return serializer_class


class AutoPermissionMixin(object):
    def get_permissions(self):
        logging.info("AutoPermissionMixin.get_permissions")
        permissions = [permission() for permission in self.permission_classes]
        if self.permissions is not None:
            permission_class_name = "{model_name}Permission".format(
                model_name=self.model.__name__)
            if hasattr(self.permissions, permission_class_name):
                permissions.append(
                    getattr(self.permissions, permission_class_name)()
                )
        else:
            logging.info("no permissions file")
        return permissions



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Xiang Wang @ 2019-10-10 22:28:41


from django.db.models import Model
from django.db.models.fields.related import ManyToManyField

from rest_framework import serializers

import django_filters

from rest_extensions.fields import DataListField
from rest_extensions.serializers import MyJSONField


class CreateSerializerFactory(object):
    def __init__(self, model):
        self.all_fields = model._meta.get_fields()
        self.simple_field = model._meta.fields
        self.model = model

        class Serializer(serializers.ModelSerializer):
            class Meta:
                fields = []

            def create(self, validated_data):
                poped_many_fields = {}  # {"name": [1,2,3] }
                for many_field in set(self.Meta.model._meta.get_fields()) - set(self.Meta.model._meta.fields):
                    if many_field.name in validated_data:
                        poped_many_fields[many_field.name] = validated_data.pop(many_field.name)
                instance = self.Meta.model.objects.create(**validated_data)
                for field_name, values in poped_many_fields.items():
                    getattr(instance, field_name).set(values)
                    instance.save()
                return instance

        self.serializer_class = Serializer
        self.serializer_class.Meta.model = self.model
        self.serializer_class.serializer_field_mapping[
            DataListField] = MyJSONField

    def create_serializer(self):
        for field in self.all_fields:
            if field in self.simple_field:
                self.serializer_class.Meta.fields.append(field.name)
            elif isinstance(field, ManyToManyField):
                _ = serializers.PrimaryKeyRelatedField(
                    queryset=field.related_model.objects.all(), many=True)
                setattr(self.serializer_class, field.name, _)
                self.serializer_class.Meta.fields.append(field.name)
        return self.serializer_class


class ListSerializerFactory(object):
    def __init__(self, model):
        self.all_fields = model._meta.get_fields()
        self.simple_field = model._meta.fields
        self.model = model

        class Serializer(serializers.ModelSerializer):
            class Meta:
                fields = []

            def create(self, validated_data):
                poped_many_fields = {}  # {"name": [1,2,3] }
                instance = self.Meta.model.objects.create(**validated_data)
                for field_name, values in poped_many_fields.items():
                    getattr(instance, field_name).set(values)
                    instance.save()
                return instance

        self.serializer_class = Serializer
        self.serializer_class.Meta.model = self.model
        self.serializer_class.serializer_field_mapping[
            DataListField] = MyJSONField

    def create_serializer(self):
        for field in self.all_fields:
            if field in self.simple_field:
                self.serializer_class.Meta.fields.append(field.name)
        return self.serializer_class


class RetrieveSerializerFactory(object):
    def __init__(self, model):
        self.all_fields = model._meta.get_fields()
        self.simple_field = model._meta.fields
        self.model = model

        class Serializer(serializers.ModelSerializer):
            class Meta:
                fields = []
                depth = 2

        self.serializer_class = Serializer
        self.serializer_class.Meta.model = self.model
        self.serializer_class.serializer_field_mapping[
            DataListField] = MyJSONField

    def create_serializer(self):
        for field in self.simple_field:
            self.serializer_class.Meta.fields.append(field.name)
        return self.serializer_class


class UpdateSerializerFactory(object):
    def __init__(self, model):
        self.all_fields = model._meta.get_fields()
        self.simple_field = model._meta.fields
        self.model = model

        class Serializer(serializers.ModelSerializer):
            class Meta:
                fields = []

            def update(self, instance, validated_data):
                all_fields = self.Meta.model._meta.get_fields()
                simple_field = self.Meta.model._meta.fields
                poped_many_fields = {}  # {"name": [1,2,3] }
                for field in all_fields:
                    if field in simple_field:
                        continue
                    elif isinstance(field, ManyToManyField):
                        if field.name not in validated_data:
                            continue
                        else:
                            poped_many_fields[field.name] = validated_data.pop(
                                field.name)
                for field_name, values in poped_many_fields.items():
                    getattr(instance, field_name).set(values)
                    instance.save()
                for attr, value in validated_data.items():
                    setattr(instance, attr, value)
                instance.save()
                return instance

        self.serializer_class = Serializer
        self.serializer_class.Meta.model = self.model
        self.serializer_class.serializer_field_mapping[
            DataListField] = MyJSONField

    def create_serializer(self):
        for field in self.all_fields:
            if field.name == "user":
                continue
            elif field in self.simple_field:
                self.serializer_class.Meta.fields.append(field.name)
            elif isinstance(field, ManyToManyField):
                _ = serializers.PrimaryKeyRelatedField(
                    queryset=field.related_model.objects.all(), many=True)
                setattr(self.serializer_class, field.name, _)
                self.serializer_class.Meta.fields.append(field.name)
        return self.serializer_class


class FilterClassFactory(object):
    def __init__(self, model):
        self.all_fields = model._meta.get_fields()
        self.simple_field = model._meta.fields
        self.model = model

        class FilterClass(django_filters.rest_framework.FilterSet):
            class Meta:
                fields = ["id"]

        self.filter_class = FilterClass
        self.filter_class.Meta.model = self.model

    def create_filter_class(self):
        for field in self.all_fields:
            self.filter_class.Meta.fields.append(field.name)
        return self.filter_class


def document_tuple(choices):
    return "; ".join(map(lambda x: u"%s:%s" % (x[0], x[1]), choices))


def get_fields(model_class: Model) -> list:
    result = []
    for field in model_class._meta.fields:
        data = {
            "name": field.name,
            "verbose_name": field.verbose_name,
            "help_text": field.help_text,
            "blank": field.blank,
            "type": {
                "BooleanField": "bool",
                "IntegerField": "integer",
                "BigAutoField": "integer",
                "TextField": "text",
            }[field.__class__.__name__]
        }
        # if isinstance(field, Model.IntegerFields):
        result.append(data)
    return result



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Xiang Wang @ 2019-10-17 09:31:03


import random
import logging
from django.db.models import Field, TextField
import json


log = logging.getLogger(__name__)


class DataListField(TextField):

    def from_db_value(self, value, expression, connection):
        if not value:
            return []
        if isinstance(value, list):
            return value
        try:
            return json.loads(value)
        except Exception as e:
            log.error(e)
            return []

    def to_python(self, value):
        log.debug("DataListField.to_python")
        if not value:
            return []
        if not isinstance(value, str):
            return value
        try:
            return json.loads(value)
        except Exception as e:
            log.error(e)
            return []

    def get_prep_value(self, value):
        log.debug("DataListField.get_prep_value")
        log.debug(value)
        if not value:
            return json.dumps([])
        if isinstance(value, str):
            return value
        return json.dumps(value, indent=4)

    def value_to_string(self, obj):
        if isinstance(obj, str):
            return obj
        else:
            return json.dumps(obj, indent=4)


class DictField(Field):

    description = "enable mysql or sqlite to store json data"

    def to_python(self, value):
        return json.loads(value)

    def from_db_value(self, value, expression, connection):
        return json.loads(value)


class Color:
    """color"""

    def __init__(self, r, g, b):
        self.r = r
        self.g = g
        self.b = b

    @classmethod
    def default(cls):
        return cls(255, 255, 255)

    @classmethod
    def random(cls):
        return cls(
            random.randint(0, 255),
            random.randint(0, 255),
            random.randint(0, 255),
            )

    def __eq__(self, color):
        if isinstance(color, Color):
            return self.r == color.r and self.g == color.g and self.b == color.b
        return False

    def __str__(self):
        return "r: {}, g: {}, b: {}".format(self.r, self.g, self.b)

    def __repr__(self):
        return "Color: {}".format(self)


class ColorField(Field):

    def __init__(self, *args, **kwargs):
        kwargs["max_length"] = 52
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        log.debug("ColorField.deconstruct")
        name, path, args, kwargs = super().deconstruct()
        del kwargs["max_length"]
        return name, path, args, kwargs

    def db_type(self, connection):
        log.debug("ColorField.db_type")
        return "char(52)"

    def from_db_value(self, value, expression, connection):
        log.info("ColorField.from_db_value")
        if value is None:
            return Color.default()
        r, g, b = list(map(lambda x: int(x), value.split(",")))
        return Color(r, g, b)

    def to_python(self, value):
        log.info("ColorField.to_python")
        if isinstance(value, Color):
            return value
        if value is None:
            return Color.default()
        r, g, b = list(map(lambda x: int(x), value.split(",")))
        return Color(r, g, b)

    def get_prep_value(self, value):
        log.debug("ColorField.get_prep_value")
        log.debug(value)
        if value is None:
            return ",".join(["255", "255", "255"])
        return ",".join(map(lambda x: str(x), [value.r, value.g, value.b]))



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Xiang Wang @ 2019-10-11 11:59:22


import json
from rest_framework import serializers


class MyJSONField(serializers.Field):
    default_error_messages = {
        'invalid': 'Value must be valid JSON.'
    }

    def to_internal_value(self, data):
        if not data:
            return data
        if isinstance(data, dict):
            return json.dumps(
                data, indent=4, ensure_ascii=False, sort_keys=True)
        elif isinstance(data, list):
            return json.dumps(
                data, indent=4, ensure_ascii=False, sort_keys=True)
        else:
            self.fail('invalid')

    def to_representation(self, value):
        try:
            return json.loads(value)
        except:
            return value


class MultiDeleteSerializer(serializers.Serializer):
    ids = serializers.ListField(child=serializers.IntegerField(), help_text="要删除的id列表")

    class Meta:
        fields = ["ids"]


class MultiActionSerializer(serializers.Serializer):
    ids = serializers.ListField(child=serializers.IntegerField(), help_text="要操作的id列表")

    class Meta:
        fields = ["ids"]



from django.db.models.fields.files import FileField
from django.utils.decorators import classonlymethod
from functools import update_wrapper
import importlib
import json
import logging
from django.views.decorators.csrf import csrf_exempt
from django.contrib.admin.models import LogEntry
from django.contrib.contenttypes.models import ContentType
from rest_framework import viewsets
from rest_framework.exceptions import PermissionDenied
from rest_framework.generics import (
    GenericAPIView, ListCreateAPIView, RetrieveUpdateDestroyAPIView
)
from rest_framework.response import Response
from rest_framework.serializers import ModelSerializer
from . import paginations, utils, mixins
from .serializers import MultiDeleteSerializer
from .filters import create_filter


class MyModelViewSet(mixins.AutoPermissionMixin, mixins.AutoSerializerMixin,
                     viewsets.ModelViewSet):
    model = None
    queryset = None
    pagination_class = paginations.PaginationClass
    permission_classes = []
    filters = None

    def __init__(self, *args, **kwargs):
        if not hasattr(self, "filter_class"):
            self.filter_class = create_filter(self.filters, self.model)
        return super(MyModelViewSet, self).__init__(*args, **kwargs)

    def get_queryset(self):
        return self.model.objects.all()

    def list(self, request, *args, **kwargs):
        logging.info("查看列表")
        logging.info(self.filter_class)
        return super(MyModelViewSet, self).list(request, *args, **kwargs)

    @classonlymethod
    def as_view(cls, actions=None, **initkwargs):
        """
        Because of the way class based views create a closure around the
        instantiated view, we need to totally reimplement `.as_view`,
        and slightly modify the view function that is created and returned.
        """
        # The name and description initkwargs may be explicitly overridden for
        # certain route configurations. eg, names of extra actions.
        cls.name = None
        cls.description = None

        # The suffix initkwarg is reserved for displaying the viewset type.
        # This initkwarg should have no effect if the name is provided.
        # eg. 'List' or 'Instance'.
        cls.suffix = None

        # The detail initkwarg is reserved for introspecting the viewset type.
        cls.detail = None

        # Setting a basename allows a view to reverse its action urls. This
        # value is provided by the router through the initkwargs.
        cls.basename = None

        # actions must not be empty
        if not actions:
            raise TypeError("The `actions` argument must be provided when "
                            "calling `.as_view()` on a ViewSet. For example "
                            "`.as_view({'get': 'list'})`")

        # sanitize keyword arguments
        for key in initkwargs:
            if key in cls.http_method_names:
                raise TypeError("You tried to pass in the %s method name as a "
                                "keyword argument to %s(). Don't do that." %
                                (key, cls.__name__))
            if not hasattr(cls, key):
                raise TypeError("%s() received an invalid keyword %r" %
                                (cls.__name__, key))

        # name and suffix are mutually exclusive
        if 'name' in initkwargs and 'suffix' in initkwargs:
            raise TypeError(
                "%s() received both `name` and `suffix`, which are "
                "mutually exclusive arguments." % (cls.__name__))

        def view(request, *args, **kwargs):
            """文档"""
            self = cls(**initkwargs)
            # We also store the mapping of request methods to actions,
            # so that we can later set the action attribute.
            # eg. `self.action = 'list'` on an incoming GET request.
            self.action_map = actions

            # Bind methods to actions
            # This is the bit that's different to a standard view
            for method, action in actions.items():
                handler = getattr(self, action)
                setattr(self, method, handler)

            if hasattr(self, 'get') and not hasattr(self, 'head'):
                self.head = self.get

            self.request = request
            self.args = args
            self.kwargs = kwargs

            # And continue as usual
            return self.dispatch(request, *args, **kwargs)

        # take name and docstring from class
        # class TmpClass(object):
        #     pass
        # tmpclass = TmpClass
        # tmpclass.__doc__ = "文档"
        # update_wrapper(view, tmpclass, updated=())
        # update_wrapper(view, cls, updated=())

        # and possible attributes set by decorators
        # like csrf_exempt from dispatch
        update_wrapper(view, cls.dispatch, assigned=())

        # We need to set these on the view function, so that breadcrumb
        # generation can pick out these bits of information from a
        # resolved URL.
        view.cls = cls
        view.initkwargs = initkwargs
        view.actions = actions
        return csrf_exempt(view)

    def perform_create(self, serializer):
        kwargs = {}
        if hasattr(self.model, "user"):
            kwargs['user'] = self.request.user
            if self.request.user.is_authenticated is False:
                raise PermissionDenied("请先登录")
            if hasattr(self.model, "last_editor"):
                kwargs['last_editor'] = self.request.user
            instance = serializer.save(**kwargs)
            contenttype = ContentType.objects.get_for_model(self.model)
            LogEntry.objects.create(user=self.request.user,
                                    object_id=instance.id,
                                    object_repr=str(instance),
                                    content_type=contenttype,
                                    change_message=json.dumps([{
                                        'added': {}
                                    }]),
                                    action_flag=1)
        else:
            serializer.save()

    def get_filter_fields(self):
        filter_fields = []
        for field in self.model._meta.fields:
            if isinstance(field, FileField):
                continue
            filter_fields.append(field.name)
        return filter_fields

    def perform_destroy(self, instance):
        contenttype = ContentType.objects.get_for_model(self.model)
        LogEntry.objects.create(user=self.request.user,
                                object_id=instance.id,
                                object_repr=str(instance),
                                content_type=contenttype,
                                change_message="",
                                action_flag=3)
        instance.delete()

    def perform_update(self, serializer):
        if hasattr(self.model, "last_editor"):
            kwargs = {"last_editor": self.request.user}
        else:
            kwargs = {}
        instance = serializer.save(**kwargs)
        contenttype = ContentType.objects.get_for_model(self.model)
        LogEntry.objects.create(user=self.request.user,
                                object_id=instance.id,
                                object_repr=str(instance),
                                content_type=contenttype,
                                change_message="[]",
                                action_flag=2)
        return instance

    def update(self, request, *args, **kwargs):
        logging.info("update")
        model = self.model
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        for field in model._meta.get_fields():
            logging.info(field)
            if field.name not in request.data:
                continue
            else:
                logging.info(field)
        serializer = self.get_serializer(instance,
                                         data=request.data,
                                         partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        serializer_class = utils.RetrieveSerializerFactory(
            model).create_serializer()
        data = serializer_class(instance).data
        return Response(data)

    def create(self, request, *args, **kwargs):
        model = self.model
        for field in model._meta.get_fields():
            if field.name not in request.data:
                continue
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        return Response(serializer.data, status=201)


class MultiDeleteView(GenericAPIView):
    serializer_class = MultiDeleteSerializer

    def post(self, request, *args, **kwargs):
        """批量删除的接口"""
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.queryset.filter(id__in=serializer.validated_data["ids"]).delete()
        return Response(status=204)


class ModelViewSetFactory(object):
    def __init__(self, app_module, model, app_config=None):
        self.model = model
        self.app_module = app_module
        self.app_config = app_config

    def create_view_set(self):
        logging.info("create_view_set")

        class TmpModelViewSet(MyModelViewSet):
            pass

        model_view_set = TmpModelViewSet
        model_view_set.model = self.model
        model_view_set.queryset = self.model.objects.all()
        model_view_set.app_module = self.app_module
        model_view_set.app_config = self.app_config
        model_view_set.permissions = getattr(self.app_module, "permissions",
                                             None)
        model_view_set.serializers = getattr(self.app_module, "serializers",
                                             None)

        filter_class = create_filter(
            model_view_set.filters, model_view_set.model)
        print("开始找filters")
        print(self.app_config.name)
        if self.app_config.name == "money":
            import ipdb
            # ipdb.set_trace();
        if importlib.util.find_spec(
                "{}.filters".format(self.app_config.name)):
            print("has filters")
            filters = importlib.import_module(
                "{}.filters".format(self.app_config.name))
            filter_name = "{}Filter".format(self.model.__name__)
            if hasattr(filters, filter_name):
                filter_class = getattr(filters, filter_name)
                print("filter_class变了")
        model_view_set.filter_class = filter_class
        if self.app_config.name == "money":
            logging.info("money的filter_class是")
            logging.info(filter_class)
        return model_view_set


def get_list_create_api_view(model_class):
    class _ListCreateAPIView(ListCreateAPIView):

        ordering = ["id"]

        def get_queryset(self):
            return model_class.objects.all()

        def get_serializer_class(self):
            class Serializer(ModelSerializer):
                class Meta:
                    model = model_class
                    fields = "__all__"
            return Serializer
    return _ListCreateAPIView


def get_detail_api_view(model_class):
    class _RetrieveUpdateDestroyAPIView(RetrieveUpdateDestroyAPIView):

        ordering = ["id"]

        def get_queryset(self):
            return model_class.objects.all()

        def get_serializer_class(self):
            class Serializer(ModelSerializer):
                class Meta:
                    model = model_class
                    fields = "__all__"
            return Serializer

        def post(self, request, *args, **kwargs):
            return self.patch(request, *args, **kwargs)
    return _RetrieveUpdateDestroyAPIView



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Xiang Wang @ 2019-11-05 18:06:29


import logging
from rest_framework import routers
from django.apps import apps
from django.urls import path, include
from . import views

log = logging.getLogger(__name__)


log.debug("load rest_extensions urls")
log.debug(apps.app_configs)
app_name = "rest_extensions"
router = routers.DefaultRouter()
api_urlpatterns = []


class AppUrls(object):
    """the url for all app"""

    def __init__(self, app_config):
        self.urlpatterns = []
        self.app_name = app_config.name
        self.app_module = app_config.module
        for model in app_config.get_models():
            model_view_set = views.ModelViewSetFactory(
                self.app_module, model, app_config).create_view_set()
            router.register(model.__name__, model_view_set)
            self.urlpatterns.append(
                path("{}/multidelete/".format(model.__name__),
                     views.MultiDeleteSerializer),
            )
        self.urlpatterns += router.urls


class AppModelUrl(object):
    pass


for app_key_name in apps.app_configs:
    log.debug(app_key_name)
    app_config = apps.app_configs[app_key_name]
    app_urls = AppUrls(app_config)
    log.debug("load app: {}".format(app_key_name))
    api_urlpatterns.append(
        path("{}/".format(app_key_name), include(
            app_urls, namespace=app_key_name)),
    )
    log.debug(app_urls.urlpatterns)

app_name = "rest_extensions"
urlpatterns = list(api_urlpatterns)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Xiang Wang @ 2017-10-12 11:25:13

from rest_framework import pagination


class PaginationClass(pagination.PageNumberPagination):
    page_size = 10
    page_size_query_param = 'page_size'
    max_page_size = 1000



import sys

from django.core.management.base import OutputWrapper
from django.core.management.color import color_style
from django.test import TestCase
from django.contrib.auth import get_user_model
from rest_extensions.fields import Color


out = OutputWrapper(sys.stdout)
style = color_style()
User = get_user_model()


def head(text):
    out.write(style.SQL_TABLE("# {}".format(text)))

def head1(text):
    out.write(style.MIGRATE_HEADING("## {}".format(text)))

def list1(text):
    out.write(style.SQL_FIELD("* {}".format(text)))

def list2(text):
    out.write(style.SQL_COLTYPE("    * {}".format(text)))

def info(text):
    out.write(style.HTTP_INFO(text))

def success(text):
    out.write(style.SUCCESS(text))


head("test rest_extensions")
class ColorFieldTestCase(TestCase):

    def setUp(self):
        head1("test ColorField")
        self.user = User.objects.create(username="testuser")

    def test_color_field(self):
        list1("test default color field")
        info(self.user.fav_color)
        self.assertEqual(self.user.fav_color, Color(255, 255, 255))
        list1("test save color")
        rand_color = Color.random()
        self.user.fav_color = rand_color
        self.user.save()
        user = User.objects.get(id=self.user.id)
        self.assertEqual(user.fav_color, rand_color)
        list1("test search color")
        rand_color = Color.random()
        User.objects.create(fav_color=rand_color)
        info(rand_color)
        self.assertEqual(User.objects.count(), 2)
        self.assertEqual(User.objects.filter(fav_color=rand_color).count(), 1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Xiang Wang @ 2019-02-18 15:41:36

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-rest-extensions",
    version="0.2.0",
    author="Xiang Wang",
    author_email="ramwin@qq.com",
    description="create normal api for all your models",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ramwin/django-rest-extensions",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "django>=2",
        "djangorestframework",
    ],
)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Xiang Wang @ 2019-10-11 17:18:09


import logging
from django.db.models import Q
from django.db.models.fields import DateTimeField, CharField, TextField
from django.db.models.fields.files import FileField
from django.db.models.fields.related import ForeignKey
from django.contrib.auth import get_user_model
import django_filters


def create_filter(filters, model):
    if filters is not None:
        filter_name = "{}Filter".format(model.__name__.capitalize())
        if hasattr(filters, filter_name):
            return getattr(filters, filter_name)

    class Meta:
        fields = {}

    tmpMeta = Meta
    tmpMeta.model = model
    for field in model._meta.fields:
        if isinstance(field, FileField):
            continue
        elif isinstance(field, ForeignKey):
            related_model = field.related_model
            tmpMeta.fields[field.name] = ["exact"]
            for foreignkey_field in related_model._meta.fields:
                if isinstance(foreignkey_field, CharField):
                    new_field_name = "{}__{}".format(field.name, foreignkey_field.name)
                    tmpMeta.fields[new_field_name] = ["icontains", "exact"]
        elif isinstance(field, DateTimeField):
            tmpMeta.fields[field.name] = ["lte", "gte"]
        elif isinstance(field, TextField):
            tmpMeta.fields[field.name] = ["exact", "icontains"]
        elif isinstance(field, CharField):
            tmpMeta.fields[field.name] = ["exact", "icontains"]
        else:
            tmpMeta.fields[field.name] = ["exact"]
    tmpMeta.fields["keyword"] = []

    class ModelFilter(django_filters.rest_framework.FilterSet):
        keyword = django_filters.CharFilter(label="keyword", help_text="keyword关键词过滤", method="filter_keyword")

        Meta = tmpMeta

        def filter_keyword(self, queryset, name, value):
            q = Q()
            for field in queryset.model._meta.fields:
                if field.name == 'password':
                    continue
                if isinstance(field, CharField) or isinstance(field, TextField):
                    kwargs = {
                        "{}__icontains".format(field.name): value
                    }
                    q |= Q(**kwargs)
            # for field in queryset.model
            return queryset.filter(q)

    tmpFilter = ModelFilter
    return tmpFilter
